<?php
/**
 * $Horde: forwards/lib/Driver/sql.php,v 1.17 2005/01/03 14:34:49 jan Exp $
 *
 * Copyright 2001-2005 Ilya Krel and Mike Cochrane
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/licenses/bsdl.php.
 *
 * Forwards_Driver_sql:: implements the Forwards_Driver API for SQL servers.
 *
 * @author  Ilya Krel <mail@krel.org.org>
 * @author  Mike Cochrane <mike@graftonhall.co.nz>
 * @version $Revision: 1.17 $
 * @since   Forwards 2.1
 * @package Forwards
 */
class Forwards_Driver_sql extends Forwards_Driver {

    /** file pointer to the sql connection. */
    var $_db;

    /** boolean which contains state of sql connection */
    var $_connected = false;

    /**
     * Constructs a new sql Forwards_Driver object.
     *
     * @param array  $params        A hash containing connection parameters.
     */
    function Forwards_Driver_sql($params = null)
    {
        if (is_null($params)) {
            $params = Horde::getDriverConfig('', 'sql');
        }
        parent::Forwards_Driver($params);
    }

    /**
     * Disconnect from the SQL server and clean up the connection.
     *
     * @return boolean true on success, false on failure.
     */
    function _disconnect()
    {
        if ($this->_connected) {
            $this->_connected = false;
            return $this->_db->disconnect();
        }

        return true;
    }

    /**
     * Begins forwarding of mail for a user.
     *
     * @param string        $user     The username to enable forwarding for.
     * @param string        $realm    The realm of the user.
     * @param string        $pass     The password of the user.
     *
     * @param string        $target   The email address that mail should
     *                                be forwarded to.
     *
     * @param optional boolean $keeplocal A flag that if true causes a
     *                                    copy of forwarded email to be
     *                                     kept in the local mailbox.
     *
     * @return boolean  Returns true on success, false on error.
     */
    function enableForwarding($user, $realm, $password, $target, $keeplocal)
    {
        // _connect() will die with Horde::fatal() upon failure.
        $this->_connect();
        $local_target=$user . '@' . $realm;

        // Build username.
        $myuser = $this->_buildUsername($user, $realm);

        // Build the SQL query.
        $query = 'UPDATE ' . $this->_params['table'];
        $query .= ' SET ' . $this->_params['forward'] . ' = ' . $this->_db->quote("y");
        $query .= ' WHERE ' . $this->_params['user_col'] . ' = ' . $this->_db->quote($myuser);
        $query .= ' AND ' . $this->_params['pass_col'] . ' = ' . $this->_db->quote(md5($password));

        $query1 = 'UPDATE ' . $this->_params['table'];
        if ($keeplocal === 'on') {
            if(!strstr($target,$local_target)) {
                $query1 .= ' SET ' . $this->_params['altemail'] . ' = ' .
                    $this->_db->quote($target . ',' . $local_target);
            }
        } else {
            $query1 .= ' SET ' . $this->_params['altemail'] . ' = ' . $this->_db->quote($target);
        }
        $query1 .= ' WHERE ' . $this->_params['user_col'] . ' = ' . $this->_db->quote($myuser);
        $query1 .= ' and ' . $this->_params['pass_col'] . ' = ' . $this->_db->quote(md5($password));

        // Execute the query.
        $result = $this->_db->query($query);
        $result1 = $this->_db->query($query1);

        if (!is_a($result, 'PEAR_Error') && !is_a($result1, 'PEAR_Error')) {
            if ($result === DB_OK && $result1 === DB_OK) {
                $this->_disconnect();
                return true;
            }
        }

        $this->_disconnect();
        return false;
    }

    /**
     * Stops forwarding of mail for a user.
     *
     * @param string        $user      The username of the user.
     * @param string        $realm    The realm of the user.
     * @param string        $pass      The password of the user.
     *
     * @return boolean  Returns true on success, false on error.
     */
    function disableForwarding($user, $realm, $password)
    {
        // _connect() will die with Horde::fatal() upon failure.
        $this->_connect();

        // Build username.
        $myuser = $this->_buildUsername($user, $realm);

        // Build the SQL query.
        $query = 'UPDATE ' . $this->_params['table'];
        $query .= ' SET ' . $this->_params['forward'] . ' = ' . $this->_db->quote('n');
        $query .= ' ,  ' . $this->_params['altemail'] . ' = ' . $this->_db->quote('');
        $query .= ' WHERE ' . $this->_params['user_col'] . ' = ' . $this->_db->quote($myuser);
        $query .= ' AND ' . $this->_params['pass_col'] . ' = ' . $this->_db->quote(md5($password));

        // Execute the query.
        $result = $this->_db->query($query);

        if (!is_a($result, 'PEAR_Error')) {
            if ($result === DB_OK) {
                $this->_disconnect();
                return true;
            }
        }

        $this->_disconnect();
        return false;
    }

    /**
     * Retrieves status of mail redirection for a user
     *
     * @param string        $user      The username of the user to check.
     *
     * @param string        $realm    The realm of the user to check.
     *
     * @return mixed        Returns 'Y' if forwarding is enabled, or false.
     */
    function isEnabledForwarding($user, $realm, $password)
    {
        // Build username.
        $myuser = $this->_buildUsername($user, $realm);

        // get current details
        $current_details = $this->_getUserDetails($myuser, $realm, $password);
        if ($current_details === false) {
            return false;
        }

        // check forwarding flag
        if ($current_details[$this->_params['forward']] === 'y' ||
             $current_details[$this->_params['forward']] === 'Y') {
            return 'Y';
        } else {
            return false;
        }
    }

    /**
     * Returns true if a local copy of forwarded messages is being kept
     *
     * @param string        $user      The username of the user to check.
     *
     * @param string        $realm    The realm of the user to check.
     *
     * @return boolean      Returns true if retain local copy is enabled
     *                      else false.
     */
    function isKeepLocal($user, $realm, $password)
    {
        // Build username.
        $myuser = $this->_buildUsername($user, $realm);

        // Get current details.
        $current_details = $this->_getUserDetails($myuser, $realm, $password);
        if ($current_details === false) {
            return false;
        }

        // Check retain copy flag.
        if (substr_count($current_details[$this->_params['altemail']], $user . '@' . $realm)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Retrieves current target of mail redirection
     *
     * @param string        $user      The username of the user.
     * @param string        $realm    The realm of the user.
     *
     * @return string   A string of current forwarding address or false.
     */
    function currentTarget($user, $realm, $password)
    {
        $searchString = ",$user@$realm";
        $current_details = $this->_getUserDetails($user, $realm, $password);

        // Check current forwarding mail address.
        $targets = $current_details[$this->_params['altemail']];
        return str_replace($searchString, "", $current_details[$this->_params['altemail']]);
    }

    /**
     * Retrieves user details from the backend.
     *
     * @param string $user      The username for which to retrieve details.
     * @param string $realm     The realm (domain) for the user.
     * @param string $password  The password for user.
     *
     * @return array|bool  Hash with user details or false.
     */
    function _getUserDetails($user, $realm, $password)
    {
        static $row;

        // If we already have the details, return now.
        is (isset($row)) {
            return $row;
        }

        // _connect() will die with Horde::fatal() upon failure.
        $this->_connect();

        // Build username.
        $myuser = $this->_buildUsername($user, $realm);

        // Build the SQL query.
        $query = 'SELECT * FROM ' . $this->_params['table'];
        $query .= ' WHERE ' . $this->_params['user_col'] . ' = ' . $this->_db->quote($myuser);
        $query .= ' AND password = ' . $this->_db->quote(md5($password));

        // Execute the query.
        $result = $this->_db->query($query);

        if (!is_a($result, 'PEAR_Error')) {
            $row = $result->fetchRow(DB_FETCHMODE_ASSOC);

            if (is_array($row)) {
                $this->_disconnect();
                return $row;
            } else {
                $this->_disconnect();
                $result->free();
                return false;
            }
        }

        $this->_disconnect();
        return false;
    }

    /**
     * Builds a username based on presence of realm.
     *
     * @return string  $user@$realm or $user.
     */
    function _buildUsername($user, $realm)
    {
        if ($realm === 'default' ||
            $realm === '') {
            return $user;
        } else {
            return $user . '@' . $realm;
        }
    }

    /**
     * Do an sql connect and login as user with privilege to change
     * passwd.
     *
     * @return   boolean   True or False based on success of connect
     */
    function _connect()
    {
        if (!$this->_connected) {
            Horde::assertDriverConfig($this->_params, 'server',
                array('phptype', 'hostspec', 'username', 'database', 'table'),
                'authentication SQL');

            // Connect to SQL server using supplied parameters.
            include_once 'DB.php';
            $this->_db = &DB::connect($this->_params,
                                      array('persistent' => !empty($this->_params['persistent'])));
            if (is_a($this->_db, 'PEAR_Error')) {
                Horde::fatal(PEAR::raiseError(_("Unable to connect to SQL server.")), __FILE__, __LINE__);
            }

            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);

            $this->_connected = true;
        }

        return true;
    }

}
